-- Broker_AltGuild by yess, yessica@fantasymail.de
local guildname = nil

local ldb = LibStub:GetLibrary("LibDataBroker-1.1",true)
local wholib = LibStub:GetLibrary('LibWho-2.0'):Library()

local QTC = LibStub('LibQTipClick-1.1')

local players, dataobj, tooltip
local update = false
local delay = 65
local counter = 0
local optionPanel

local function Debug(...)
	local s = "Broker_AltGuild Debug:"
	for i=1,select("#", ...) do
		local x = select(i, ...)
		s = strjoin(" ",s,tostring(x))
	end
	DEFAULT_CHAT_FRAME:AddMessage(s)
end

local function RGBToHex(r, g, b)
	return ("%02x%02x%02x"):format(r*255, g*255, b*255)	
end

local function UpdateText(event, results, a2)
	players = {} --results
	count = 0
	for k,v in pairs(results) do
		if strupper(v.Guild) == strupper(guildname) then
			count = count +1
			table.insert(players, v)
		end
	end
	dataobj.text = count
end

local function updateList()
	if guildname then
		wholib:Who('g-' .. guildname, {queue = wholib.WHOLIB_QUERY_QUIET, timeout = 0, callback = UpdateText})
	else
		dataobj.text = "no guild set"
	end
end

local function setGuild(guild)
	guildname = guild
	AltGuildDB = guild
	updateList()
	DEFAULT_CHAT_FRAME:AddMessage("Broker_AltGuild: Guild set to "..guild)
end

dataobj = ldb:NewDataObject("Broker_AltGuild", {
	type = "data source",
	icon = "Interface\\AddOns\\Broker_AltGuild\\icon",
	label = "Broker_AltGuild",
	text  = "Updating...",
	OnClick = function(self, button) 
		if button == "RightButton" then 
			InterfaceOptionsFrame_OpenToCategory(optionPanel);
		else
			updateList()
		end
	end
})

local function MouseHandler(event, cell, arg, button)
	--Debug("MouseHandler",event, cell, arg, button)
	local name = arg
	if button == "RightButton" then
		InviteUnit(name)
	else
		SetItemRef("player:"..name, "|Hplayer:"..name.."|h["..name.."|h", "LeftButton")
	end
end

local function HideTooltip()
	if not tooltip or MouseIsOver(tooltip) then return end
	tooltip:SetScript("OnLeave", nil)
	tooltip:Hide()
	QTC:Release(tooltip)
	tooltip = nil
end

local function LeaveHandler(event, cell, arg)
        QTC.OnLeave(event, cell, arg)
		HideTooltip()
end

function dataobj:OnEnter()
	-- Acquire a tooltip with 3 columns, respectively aligned to left, center and right
	tooltip = QTC:Acquire("AltGuildTooltip", 3, "LEFT", "CENTER","RIGHT")
	tooltip:Clear()
	self.tooltip = tooltip 
	tooltip:SetCallback("OnMouseDown", MouseHandler)
	tooltip:SetCallback("OnLeave", LeaveHandler)
	--tooltip:SetAutoHideDelay(0.2, self)
	
	tooltip:AddNormalLine(guildname or "type /brokerag <guildname> to set the guild to watch.")
	tooltip:AddNormalLine("")
	tooltip:AddNormalLine("Name", "Level","Zone")
	tooltip:AddSeparator(1,1,1,1,0.5)
	
	if players then
		for k,v in pairs(players) do
			--if v.Guild == guildname then
				--Debug(v.Guild)
				local classcolor = RAID_CLASS_COLORS[v.NoLocaleClass:upper()] or {r=0.77, g=0.12 ,b=0.23 }
				local y, x = tooltip:AddLine()
				y, x = tooltip:SetCell(y, 1, "|cff"..RGBToHex(classcolor.r, classcolor.g, classcolor.b)..v.Name,v.Name)
				y, x = tooltip:SetCell(y, 2, v.Level ,v.Name)
				y, x = tooltip:SetCell(y, 3, v.Zone,v.Name)
			--end
		end
	end
	
	tooltip:AddNormalLine("")
	tooltip:AddNormalLine("")
	tooltip:AddNormalLine("|cff"..RGBToHex(0.8, 0.8, 0.8).."Click to force refresh.")
	tooltip:AddNormalLine("|cff"..RGBToHex(0.8, 0.8, 0.8).."Right click to open options.")
	
	tooltip:SmartAnchorTo(self)
	tooltip:SetScript("OnLeave", HideTooltip)
	tooltip.anchor = self
	tooltip:Show()
end

function dataobj:OnLeave()
	HideTooltip()
end

local function slashHandler(msg)
	if msg == "debug" then
		wholib:SetWhoLibDebug(true)
		DEFAULT_CHAT_FRAME:AddMessage("Broker_AltGuild: Debug enabled.")
	else
		setGuild(msg)
	end
end


local function createOptionPanel()
	optionPanel = CreateFrame( "Frame", "Broker_AltGuildPanel", UIParent );
	local title = optionPanel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetPoint("TOPLEFT", 16, -16)
	local version = GetAddOnMetadata("Broker AltGuild","Version") or ""
	--"X-Curse-Packaged-Version"
	title:SetText("Broker AltGuild "..version)

	local subtitle = optionPanel:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	subtitle:SetHeight(35)
	subtitle:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
	subtitle:SetPoint("RIGHT", optionPanel, -32, 0)
	subtitle:SetNonSpaceWrap(true)
	subtitle:SetJustifyH("LEFT")
	subtitle:SetJustifyV("TOP")

	subtitle:SetText("Enter the name of the guild to watch.")
	
	editbox = CreateFrame("EditBox",nil,optionPanel,"InputBoxTemplate")
	--editbox:SetAllPoints(footer)
	--editbox:SetPoint("TOPLEFT", optionPanel, -10,0)
	--editbox:SetPoint("BOTTOMRIGHT", optionPanel, 100,0)
	editbox:SetHeight(35)
	editbox:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 5, -15)
	editbox:SetPoint("RIGHT", optionPanel, -132, 0)
	
	editbox:SetAutoFocus(false)
	editbox:SetScript("OnEscapePressed",function(self) 
		self:ClearFocus()
	end)
	--editbox:SetFont(subtitle.font, 12)
	editbox:SetFontObject(GameTooltipTextSmall)
	editbox:SetText("test")

	optionPanel.name = "Broker AltGuild";
	optionPanel.okay = function(self) 
		setGuild(editbox:GetText())
	end
	optionPanel.refresh = function(self) 
		if AltGuildDB then
			editbox:SetText(AltGuildDB)
		end
	end
	InterfaceOptions_AddCategory(optionPanel);
end


SlashCmdList["Broker_AltGuild"] = slashHandler;
SLASH_Broker_AltGuild1 = "/Broker_AltGuild";
SLASH_Broker_AltGuild2 = "/brokerag";
createOptionPanel()

local function OnUpdate(self, elapsed)
	counter = counter + elapsed
	if counter >= delay then
		if update then
			updateList()
		end
		counter = 0
	end
end

local function OnEvent(self, event, addon)
	if event == "PLAYER_ENTERING_WORLD" then
		guildname = AltGuildDB
		update = true
		updateList()
	else
		update = flase
	end
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:RegisterEvent("PLAYER_LEAVING_WORLD")

frame:SetScript("OnEvent", OnEvent)
frame:SetScript("OnUpdate", OnUpdate)